;******************************************************************************
; THIS PROGRAM IS PROVIDED "AS IS". TI MAKES NO WARRANTIES OR
; REPRESENTATIONS, EITHER EXPRESS, IMPLIED OR STATUTORY, 
; INCLUDING ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS 
; FOR A PARTICULAR PURPOSE, LACK OF VIRUSES, ACCURACY OR 
; COMPLETENESS OF RESPONSES, RESULTS AND LACK OF NEGLIGENCE. 
; TI DISCLAIMS ANY WARRANTY OF TITLE, QUIET ENJOYMENT, QUIET 
; POSSESSION, AND NON-INFRINGEMENT OF ANY THIRD PARTY 
; INTELLECTUAL PROPERTY RIGHTS WITH REGARD TO THE PROGRAM OR 
; YOUR USE OF THE PROGRAM.
;
; IN NO EVENT SHALL TI BE LIABLE FOR ANY SPECIAL, INCIDENTAL, 
; CONSEQUENTIAL OR INDIRECT DAMAGES, HOWEVER CAUSED, ON ANY 
; THEORY OF LIABILITY AND WHETHER OR NOT TI HAS BEEN ADVISED 
; OF THE POSSIBILITY OF SUCH DAMAGES, ARISING IN ANY WAY OUT 
; OF THIS AGREEMENT, THE PROGRAM, OR YOUR USE OF THE PROGRAM. 
; EXCLUDED DAMAGES INCLUDE, BUT ARE NOT LIMITED TO, COST OF 
; REMOVAL OR REINSTALLATION, COMPUTER TIME, LABOR COSTS, LOSS 
; OF GOODWILL, LOSS OF PROFITS, LOSS OF SAVINGS, OR LOSS OF 
; USE OR INTERRUPTION OF BUSINESS. IN NO EVENT WILL TI'S 
; AGGREGATE LIABILITY UNDER THIS AGREEMENT OR ARISING OUT OF 
; YOUR USE OF THE PROGRAM EXCEED FIVE HUNDRED DOLLARS 
; (U.S.$500).
;
; Unless otherwise stated, the Program written and copyrighted 
; by Texas Instruments is distributed as "freeware".  You may, 
; only under TI's copyright in the Program, use and modify the 
; Program without any charge or restriction.  You may 
; distribute to third parties, provided that you transfer a 
; copy of this license to the third party and the third party 
; agrees to these terms by its first use of the Program. You 
; must reproduce the copyright notice and any other legend of 
; ownership on each copy or partial copy, of the Program.
;
; You acknowledge and agree that the Program contains 
; copyrighted material, trade secrets and other TI proprietary 
; information and is protected by copyright laws, 
; international copyright treaties, and trade secret laws, as 
; well as other intellectual property laws.  To protect TI's 
; rights in the Program, you agree not to decompile, reverse 
; engineer, disassemble or otherwise translate any object code 
; versions of the Program to a human-readable form.  You agree 
; that in no event will you alter, remove or destroy any 
; copyright notice included in the Program.  TI reserves all 
; rights not specifically granted under this license. Except 
; as specifically provided herein, nothing in this agreement 
; shall be construed as conferring by implication, estoppel, 
; or otherwise, upon you, any license or other right under any 
; TI patents, copyrights or trade secrets.
;
; You may not use the Program in non-TI devices.
;******************************************************************************
;   MSP430FG439- Oversample ADC12 for Higher resolution
;   This code demonstrates oversampling and averaging on ADC12
;   Voltage and temperature are displayed on the LCD after oversampling
;   The routines used are: voltage (1 uV mode), voltage2 (100 uV mode) and 
;   temperature (.01 degree resolution). These modes are selected using the
;   push buttons on the board
;   The above functions and the calibration routine is explained in the 
;   documentation of this application report
;   SW1 = P2.1, SW2 = P2.2, SW3 = P2.3   Pull-up = P4.2
;
;   Harman Grewal 
;   Texas Instruments, Inc
;   July 2005
;   Built with CCE for MSP430 Version 1.00  
;****************************************************************************** 
        .cdecls C,LIST,"msp430xG43x.h" ;  MSP430FG439 Standard Definitions
        
        .define  001h,  LED0           ; P1.0 = LED0 
        .define  002h,  PB_VOLTAGE2    ; P2.1--SW1
        .define  004h,  PB_TEMP        ; P2.2--SW2
        .define  008h,  PB_VOLT        ; P2.3--SW3

        .define  R13,   DIGITSHI                    
        .define  R14,   DIGITSLO      

        .define  R7,    IRBT                         
        .define  R8,    IROP1                        
        .define  R9,    IROP2L        
        .define  R10,   IROP2M        
        .define  R11,   IRACL         
        .define  R12,   IRACM         
;******************************************************************************
;------------------------------------------------------------------------------ 
; word & byte variables definition
            
;------------------------------------------------------------------------------ 
         .bss  Time_out, 1             ; To detect Time out
         .bss  Calmode,1               ; To perform calibration
         .bss  push_time, 1            ; Used during calibration mode
         .bss  mode, 1                 ; Switch between operating modes
         .bss  notlpm,1                ; Determine if LPM4 switch pressed

         .bss  Tempin, 1 
         .bss  Temp_mode,1             ; Determine oC or oF mode
         .bss  negflag,1               ; Determine negative Temperature
         .align
         .bss  Temp_Sym, 2             ; DF (oF), DC(oC)
         .bss  Refcal_ram, 2           ; RAM storage for Reference voltage
         .bss  Temp_slope_ram,2        ; RAM storage for Temp slope
         .bss  Temp_offset_ram,2       ; RAM storage for Temp offset
         .bss  slopeLo, 2              ; Low Word in Slope calculation
         .bss  slopeHi,2               ; High Word in Slope calculation
         .bss  voltageLo,2             ; Low Word after slope*oversampled result
         .bss  voltageHi,2             ; Hi Word after slope*oversampled result
         .bss  TempC, 2                ; Temp in oC
         .bss  TempFLo, 2              ; Low Word for TempF calculation
         .bss  TempFHi,2               ; Hi Word for TempF calculation
         .bss  ADCresult,2             ; result after oversampling
         .bss  movingAVGLo, 2          ; Lo word for moving average
         .bss  movingAVGHi,2           ; Hi word for moving average
         .bss  AVGBuffer, 2            ; Array to store ADCMEM values for 
                                       ; moving average
         .text
;------------------------------------------------------------------------------ 
;            LCD Type Definition
;------------------------------------------------------------------------------ 
;Segments definition
a      .equ 010h                 
b      .equ 020h                 
c      .equ 040h                 
d      .equ 080h                 
e      .equ 004h                 
f      .equ 001h
g      .equ 002h
Blank  .equ 000h  
DP     .equ 008h
         .align                            ; Align words on even boundry		
LCD_Tab  .byte      a+b+c+d+e+f            ; Displays "0"
         .byte      b+c                    ; Displays "1"
         .byte      a+b+d+e+g              ; Displays "2"
         .byte      a+b+c+d+g              ; Displays "3"
         .byte      b+c+f+g                ; Displays "4"
         .byte      a+c+d+f+g              ; Displays "5"
         .byte      a+c+d+e+f+g            ; Displays "6"
         .byte      a+b+c                  ; Displays "7"
         .byte      a+b+c+d+e+f+g          ; Displays "8"
         .byte      a+b+c+d+f+g            ; Displays "9"
         .byte      a+b+c+e+f+g            ; Displays "A"
         .byte      Blank                  ; Displays  Blank
         .byte      a+d+e+f                ; Displays "C"
         .byte      a+b+f+g                ; Displays "degrees" o
         .byte      a+d+e+f+g              ; Displays "E"
         .byte      a+e+f+g                ; Displays "F"
;------------------------------------------------------------------------------ 

            ;Default calibration constants
Refcal_flash  .word  25000
Temp_offset_100  .word   46800  ; Offset (468 * 100) for integer
                                            ; Calculation of (oF) upto Two decimal
                                            ; places, xxx.xx oF
;------------------------------------------------------------------------------ 
RESET       mov.w   #0A00h,SP               ; Initialize 'FG439 stackpointer
            call    #Init_Sys               ; Initialize system
            clr.w   R5                      ; Pointer for AVGBuffer update
            clr.b   &Calmode
            clr.b   &Time_out
            mov.w   #0EFB4h,&slopeLo        ; Slope for temperature calc
            mov.w   #01h,&slopeHi           ; 126900 = 1EFB4h
            mov.b   #01h,&notlpm 
            clr.b   &push_time
            clr.b   &mode
            mov.b   #1,&notlpm              ; NOT LPM4
            clr.b   &Tempin
            mov.b   #1,&Temp_mode           ; Default oF Temp_mode
            clr.b   &negflag
            clr.w   &Temp_Sym
            clr.w   &Refcal_ram
            clr.w   &Temp_slope_ram
            clr.w   &movingAVGLo
            clr.w   &movingAVGHi
            clr.w   &ADCresult
            mov.w   #0,R6
clrram      mov.w   #0,AVGBuffer(R6)        ; Clr AVGBuffer
            incd.w  R6
            cmp.w   #0200h,R6               ; Clear 512 bytes (256 words)
            jnz     clrram                  ; Loopback
            mov.w   &Refcal_flash,&Refcal_ram ; move flash value to RAM
            mov.w   &Temp_offset_100,&Temp_offset_ram
            mov.b   &P2IN,&Tempin           ; 
            and.b   #PB_TEMP,&Tempin
            mov.b   &Tempin,&Calmode         
            
;------------------------------------------------------------------------------ 
; 100 uV Voltage2_Mode
; V = (ADCresult * RefVolt)/65535
; eg; V = (ADCresult * 25000)/65535 
;------------------------------------------------------------------------------ 
voltage2    
            bic.w   #SHT0_8,&ADC12CTL0      ; Clear sampling time used for other modes
            bis.w   #SHT0_1,&ADC12CTL0      ; Sampling time for this mode
            mov.b   #INCH_3+SREF_1,&ADC12MCTL0 ; A3, Internal ref
            mov.w   #0001h,&ADC12IE         ; Enable AD12IE.0             
            bis.w   #ENC,&ADC12CTL0         ; Start Conversion
            mov.w   #TBCLR+MC_1+TBSSEL_2,&TBCTL; Clr TBR,up mode, SMCLK
            bis.b   #BTIE,&IE2              ; Enable 250ms Basic Timer interrupt.
            bis.w   #LPM0,SR                ; Enter LPM0 to take ADC samples
            bic.b   #BTIE,&IE2              ; Disable Basic Timer interrupt
            mov.w   &ADCresult,IROP1        ; for multiplication
            mov.w   &Refcal_ram, IROP2L     ; 
            mov.w   #0h,IROP2M              ; Always 0
No_Ref_Cal  call    #Mul16                  ; reference*ADCresult
            mov.w   IRACM,&ADCresult        ; Hi Word is divide by 65535
            mov.w   &ADCresult,DIGITSLO     ; 
            mov.w   #0h,DIGITSHI            ; Always 0, since 2.5000 is Max (<= 16 bits)
            call    #bin2bcd
            call    #ClearLCD
            call    #Display                ; Display Data on LCD
            mov.b   #0,&Temp_mode           
            jmp     voltage2                ; Repeat  
                                         
;------------------------------------------------------------------------------ 
; Temp_Mode
; TempFHi;TempFLo = (ADCresult*Slope*100)/65535  -  offset
; eg; TempFHi;TempFLo = (ADCresult*1269*100)/65535  -  46800
; TempC = (TempF - 3200)*5/9
;------------------------------------------------------------------------------ 
temperature 
            mov.b   #1h,&notlpm
            cmp.b   #0h,&Calmode
            jnz     No_Temp_Cal
            mov.b   #1,&Temp_mode
            call    #Temp_cal
No_Temp_Cal bic.w   #SHT0_1,&ADC12CTL0      ; Clear sampling time used for other modes
            bis.w   #SHT0_8,&ADC12CTL0      ; Sampling time for this mode
            mov.b   #INCH_10+SREF_1,&ADC12MCTL0 ; Temp Sensor A10, Internal ref        
            mov.w   #0001h,&ADC12IE         ; Enable AD12IE.0
            bis.w   #ENC,&ADC12CTL0         ; Start Conversion
            mov.w   #TBCLR+MC_1+TBSSEL_2,&TBCTL; Clr TBR,up mode, SMCLK
            bis.b   #BTIE,&IE2              ; Enable 250ms Basic Timer interrupt.
            bis.w   #LPM0,SR                ; Enter LPM0 to take ADC samples
            bic.b   #BTIE,&IE2              ; Disable Basic Timer interrupt
            mov.w   #0000h,&ADC12IE         ; Disable AD12IE.0 to Display value                          
            mov.w   &slopeLo,IROP2L
            mov.w   &slopeHi,IROP2M
            mov.w   &ADCresult,IROP1 
            call    #Mul16                  ; Hi Word of result is divide by
            mov.w   IRACM,&TempFLo          ; 65535 since greater than 16 bits
            cmp.w   #33846,&ADCresult       ; If ADCresult >= 33846, TempFHi=1
            jn      Hiresult0               ; else 0
            mov.w   #1,&TempFHi
            jmp     T3
Hiresult0   mov.w   #0,&TempFHi                      
            clr.w   R7                      ; Clear working registers
            clr.w   R8
            clr.w   R14
            clr.w   R15
T3          mov.w   &TempFLo,DIGITSLO       ;  
            mov.w   &TempFHi,DIGITSHI
;--------------------------------------------------------------------------
; The following routine calculates oF upto two decimal places
; The range of oF is: -40.00 to 185.00 oF (-40 to 85 oC)
; For this range, the range of ADCresult is: 22103-33722
; An example of two numbers is demonstrated in the comments
; eg; (ADCresult*slope*100)/65535 = 42800, offset = 46800
; oF = 42801-46800 = -3999 oF (-39.99 oF)
; The decimal is placed appropriately in the LCD display routine
; NOTE; For the temperature range of this device, ADcresult will be approx 
; in the range of 22103-33722 (which are <16bits), but this routine is expanded 
; to account for ADCresult range of 0-65536 (upto 17 bits) and considers case of
; overflow during subtract (eg: ADCresult = 0, offset = 46800; 0-46800=-46800
; will cause an overflow, since the result is less than -32768
; xxxx oC = (oF - 3200)*5/9; xxxx oC is xx.xx oC 
;----------------------------------------------------------------------------
            mov.w   #00008h,SR              ; clr SR, except GIE
            mov.w   DIGITSLO,R7             ; 42801
            clrc
            rrc.w   R7                      ; 42801/2 = 21400 (0x5398)
            addc.w  R7,R8                   ; 21400 + R8 + C = 21401 (0x5399)
             
            mov.w   &Temp_offset_ram,R14    ; 46800
            clrc
            rrc.w   R14                     ; 46800/2 = 23400 (0x5B68)
            addc.w  R14,R15                 ; 23400 + R15 + C = 23400 (0x5B68)
            sub.w   R14,R7                  ; 0x5398 - 0x5B68 = 0xF830 (-2000), N=1
            sub.w   R15,R8                  ; 0x5399 - 0x5B68 = 0xF831 (-1999), N=1
            add.w   R8,R7                   ; 0xF830 + 0xF831 = 0xF061 (-3999), C=1
            jn      tstHI                   ; N=1
            bit.w   #00100h,SR              ; if overflow
            jz      posresults
            cmp.w   #0,DIGITSHI
            jz      Hizero
            clr.w   DIGITSHI
            jmp     posresults
Hizero      mov.w   #07FFFh,R8
            dec.w   R7
            sub.w   R7,R8
            bis.w   #08000h,R8
            jmp     negresults
             
tstHI       cmp.w   #0,DIGITSHI             ; Test if DIGITSHI = 0 (<= 16 bits)
            jz      comp2                   
            clr.w   DIGITSHI
            jmp     posresults
comp2       and.w   #07FFFh,R7              ; Clr MSBit,0x7FFF & 0xF061 = 0x7061
            mov.w   #07FFFh,R8              ; R8 = 32767
            dec.w   R7                      ; 0x7060 = 28768
            sub.w   R7,R8                   ; 32767-28768 = 3999
             
            call    #ClearLCD               

negresults  mov.w   R8,&TempFLo             ; TempFLo = 3999
            mov.w   DIGITSHI,&TempFHi       ; TempFHi = 0
            bis.b   #040h,&LCDM4            ; To display "-"
            mov.b   #1,&negflag             ; Set negflag for -ve number
            jmp     subdone
             
             
posresults  mov.w   R7,&TempFLo
            mov.w   DIGITSHI,&TempFHi
            bic.b   #040h,&LCDM4            ; Clr "-" display for +ve result
            clr.b   &negflag                ; Clr negflag for -ve number
             
subdone     cmp.b   #0h,&Temp_mode          ; oC or oF mode
            jz      Cmode
            mov.w   #00dfh,&Temp_Sym        ; Display "oF"
            jmp     Done
Cmode       mov.w   #00dch,&Temp_Sym        ; Display "oC"
            cmp.b   #1,negflag              ; if TempFLo is -ve number
            jnz     chkneg
            bis.b   #040h,&LCDM4            ; To display "-"
            add.w   #3200,&TempFLo          ; -F-3200 = -(F+3200)
            jmp     solvetempC              ; calculate oC
chkneg      cmp.w   #3200,&TempFLo          ; if TempFLo < 3200
            jge     posCelcius              ; If > or =, TempC is +ve
            bis.b   #040h,&LCDM4            ; To display "-"
            mov.w   #3200,R7                ; if TempFLo < 3200, F-3200 = -(3200-F) 
            sub.w   &TempFLo,R7             ; 3200-TempFLo
            mov.w   R7,&TempFLo             ; Move from temporary storage to TempFLo
            jmp     solvetempC              ; 
posCelcius  bic.b   #040h,&LCDM4            ; Clear "-" display for +ve result
            sub.w   #3200,&TempFLo
solvetempC  mov.w   &TempFLo,IROP1          ; (TempFLo-3200)*5/9
            mov.w   #5, IROP2L              ; 
            mov.w   #0h,IROP2M
            call    #Mul16
            mov.w   #9,IROP1
            mov.w   IRACL, IROP2L
            mov.w   IRACM,IROP2M
            call    #Div32
            mov.w   IRACL,&TempC            ; TempC is solved
            
Done        bic.b   #01h,P1OUT
            cmp.b   #0h,&Temp_mode          ; Display oC or oF
            jz      doC                                   
            mov.w   &TempFLo,DIGITSLO
            mov.w   &TempFHi,DIGITSHI
            cmp.w   #2134h,&TempFLo
            jlo     T1
            bis.b   #01h,P1OUT
            jmp     T1
doC         mov.w   &TempC,DIGITSLO
            mov.w   #0h,DIGITSHI
            cmp.w   #0B80h,&TempC
            jlo     T1
            bis.b   #01h,P1OUT
T1          call    #bin2bcd                             
            swpb    DIGITSLO                ; eg 184.97 oF-->9784
            swpb    DIGITSHI                ; 0001-->0100                   
            mov.w   DIGITSHI,R12            ; Temporary storage                        
            mov.b   DIGITSLO,DIGITSHI       ; eg 0084
            add.w   R12,DIGITSHI            ; 0100+0084
            and.w   #0FF00h,DIGITSLO        ; FF00&9784= 9700       
            
            add.w   &Temp_Sym,DIGITSLO      ; Add "oF" or "oC"--97DF
            call    #Display                ; Display Data on LCD
            dec.b   &Time_out               ; Display voltage2 after 60 secs
            jnz     T2
            bis.b   #PB_VOLTAGE2,&P2IFG     ; Force interrupt
T2          jmp     temperature             ; Repeat
;------------------------------------------------------------------------------ 
; 1 uV voltage_Mode
; uV = (ADCresult * Refcal_ram * 100)/65535
; eg; uV = (ADCresult * 2500000)/65535 
;------------------------------------------------------------------------------ 
voltage     
            mov.b   #1h,&notlpm
            cmp.b   #0h,&Calmode
            jnz     No_Ref_Cal3
            call    #Ref_cal
No_Ref_Cal3 bic.w   #SHT0_8,&ADC12CTL0      ; Clear sampling time used for other modes
            bis.w   #SHT0_1,&ADC12CTL0      ; Sampling time for this mode
            mov.b   #INCH_3+SREF_1,&ADC12MCTL0; A3, Internal ref
            mov.w   #0001h,&ADC12IE         ; Enable AD12IE.0               
            bis.w   #ENC,&ADC12CTL0         ; Start Conversion
            mov.w   #TBCLR+MC_1+TBSSEL_2,&TBCTL; Clr TBR,up mode, SMCLK
            bis.b   #BTIE,&IE2              ; Enable 250 ms Basic Timer interrupt.
            bis.w   #LPM0,SR                ; Enter LPM0 to take ADC samples
            bic.b   #BTIE,&IE2              ; Disable Basic Timer interrupt
            mov.w   &Refcal_ram,R12         
            clr.w   R13
            mov.w   #100,R14                ; Move 100
            clr.w   R15
            call    #Mul32                 ; Refcal_ram*100
            mov.w   R12,&voltageLo          ; Store Hi Word of Refcal_ram*100 
            mov.w   R13,&voltageHi          ; Store Lo Word of Refcal_ram*100
            mov.w   &ADCresult,R12          ; 
            clr.w   R13
            clr.w   R14
            clr.w   R15
            mov.w   &voltageLo,R8           ; 
            mov.w   &voltageHi,R9
            clr.w   R10
            clr.w   R11
            call    #_Mul64i                ; ADCresult * Refcal_ram * 100 
            mov.w   R13,&voltageLo          ; Hi Words of result  are divide by
            mov.w   R14,&voltageHi          ; 65535 since greater than 16 bits
                                                                 
            mov.w   &voltageLo,DIGITSLO     ;           
            mov.w   &voltageHi,DIGITSHI
            call    #bin2bcd
            call    #ClearLCD            
            call    #Display                ; Display Data on LCD 
            dec.b   &Time_out               ; Display voltage2 after 60 secs
            jnz     D1
            bis.b   #PB_VOLTAGE2,&P2IFG     ; Force interrupt
            mov.b   #0,&Temp_mode
D1          jmp     voltage                 ; Repeat                              

;------------------------------------------------------------------------------ 
Mul16;
;UNSIGNED MULTIPLY SUBROUTINE; 
;IROP1 x IROP2L > IRACM|IRACL      
;UNSIGNED MULTIPLY AND ACCUMULATE SUBROUTINE;
;(IROP1 x IROP2L) + IRACM|IRACL > IRACM|IRACL  
;------------------------------------------------------------------------------ 
MPYU        clr.w   IRACL                   ; 0 > LSBs RESULT
            clr.w   IRACM                   ; 0 > MSBs RESULT
MACU        
            mov.w   #1h,IRBT                ; BIT TEST REGISTER
L2          bit.w   IRBT,IROP1              ; TEST ACTUAL BIT
            jz      L1                      ; IF 0: DO NOTHING
            add.w   IROP2L,IRACL            ; IF 1: ADD MULTIPLIER TO RESULT
            addc.w  IROP2M,IRACM
L1          rla.w   IROP2L                  ; MULTIPLIER x 2
            rlc.w   IROP2M
            rla.w   IRBT                    ; NEXT BIT TO TEST
            jnc     L2                      ; IF BIT IN CARRY: FINISHED
            ret

;------------------------------------------------------------------------------ 
Div32
; UNSIGNED DIVISION SUBROUTINE 32BIT BY 16BIT
; IROP2M|IROP2L : IROP1 > IRACL, REMAINDER IN IROP2M
; RETURN; CARRY = 0; OK CARRY = 1; QUOTIENT > 16 BITS       
;------------------------------------------------------------------------------
            clr.w   IRACL                   ; CLEAR RESULT
            mov.w   #17,IRBT                ; INITIALIZE LOOP COUNTER
DIV1        cmp.w   IROP1,IROP2M
            jlo     DIV2
            sub.w   IROP1,IROP2M
DIV2        rlc.w   IRACL
            jc      DIV4                    ; Error: result > 16 bits
            dec.w   IRBT                    ; Decrement loop counter
            jz      DIV3                    ; Is 0: terminate w/o error
            rla.w   IROP2L
            rlc.w   IROP2M
            jnc     DIV1
            sub.w   IROP1,IROP2M
            setc
            jmp     DIV2
DIV3        clrc                            ; No error, C = 0        
            
DIV4        ret                             ; Error indication in C


;------------------------------------------------------------------------------ 
_Mul64i;
; Subroutine for 16 bit x 32 bit multiply
; eg; 40000 x 2500000 = 174876E800 h
; R14= 0017, R13 = 4876, R12 = E800   
;------------------------------------------------------------------------------ 	
            push.w	R6
            push.w	R7
            sub.w	#0x10,SP
            mov.w	R12,0x0(SP)
            mov.w	R13,0x2(SP)
            mov.w	R14,0x4(SP)
            mov.w	R15,0x6(SP)
            mov.w	R8,0x8(SP)
            mov.w	R9,0xA(SP)
            mov.w	R10,0xC(SP)
            mov.w	R11,0xE(SP)
            mov.w	#0x3,R7
loop2       mov.w	R10,R11
            mov.w	R9,R10
            mov.w	R8,R9
            clr.w	R8
            clr.w	R6
loop1       mov.w	R6,R15
            rla.w	R15
            add.w	SP,R15
            mov.w	@R15,R12
            clr.w	R13
            mov.w	R7,R15
            sub.w	R6,R15
            rla.w	R15
            add.w	SP,R15
            mov.w	0x8(R15),R14
            clr.w	R15
            call	#Mul32
            add.w	R12,R8
            addc.w	R13,R9
            adc.w	R10
            adc.w	R11
            inc.w	R6
            cmp.w	R6,R7
            jge         loop1
            add.w	#0xFFFF,R7
            tst.w	R7
            jge	        loop2
            mov.w	R8,R12
            mov.w	R9,R13
            mov.w	R10,R14
            mov.w	R11,R15
            add.w	#0x10,SP
            pop.w	R7
            pop.w	R6
            ret
;------------------------------------------------------------------------------ 
Mul32;
;  
;------------------------------------------------------------------------------                                                                         
	
            push.w    R10
            push.w    R11
            mov.w     R12,R10
            mov.w     R13,R11
            clr.w     R12
            clr.w     R13
loop6       tst.w     R10
            jne       loop3
            tst.w     R11
            jeq       loop4
loop3       clrc	
            rrc.w     R11
            rrc.w     R10
            jnc       loop5
            add.w     R14,R12
            addc.w    R15,R13
loop5       rla.w     R14
            rlc.w     R15
            jne       loop6
            tst.w     R14
            jne       loop6
loop4       pop.w     R11
            pop.w     R10
            ret

;------------------------------------------------------------------------------ 
Temp_cal  ;Temperature Calibration subroutine
; Temperature can be adjusted in increments of .01,0.1 and 1 oF
; Pressing SW1 & SW2 together changes the digit to be changed
; SW1 decrements the offset (and hence increses the temperature) and 
; SW2 increments the offset (and hence decreses the temperature)
;------------------------------------------------------------------------------ 
            bic.b   #PB_VOLTAGE2+PB_TEMP+PB_VOLT,&P2IE; Disable PB interrupts
            bit.b   #PB_VOLTAGE2+PB_TEMP,&P2IN; Check if SW1 and SW2 pressed
            jnz     chkpush
            inc.b   &push_time                ; Push_time determines the digit to be changed
chkpush     cmp.b   #3,&push_time             ; 
            jnz     decvalue                  ; 
            mov.b   #0,&push_time             ; 
            
decvalue    bit.b   #PB_VOLTAGE2,&P2IN        ; 
            jnz     incvalue
            cmp.b   #0,&push_time
            jnz     comp1dig
            dec.w   &Temp_offset_ram
            jmp     return
comp1dig    cmp.b   #1,&push_time
            jnz     comp2dig
            sub.w   #10,&Temp_offset_ram
            jmp     return
comp2dig    sub.w   #100,&Temp_offset_ram
            jmp     return
                 
incvalue    bit.b   #PB_TEMP,&P2IN
            jnz     chkPB3
            cmp.b   #0,&push_time
            jnz     comp11dig
            inc.w   &Temp_offset_ram
            jmp     return
comp11dig   cmp.b   #1,&push_time
            jnz     comp22dig
            add.w   #10,&Temp_offset_ram
            jmp     return
comp22dig   add.w   #100,&Temp_offset_ram
            jmp     return
chkPB3      bit.b   #PB_VOLT,&P2IN
            jz      TCAL2            
            jmp     return                    
TCAL2       dint                            ; Disable global interrupt
            Call    #flash_erase            ; Erase info memory seg A
            xor.w   #FXKEY+WRT,&FCTL1       ; Enable FLASH write
            mov.w   &Refcal_ram,&Refcal_flash       ;
            mov.w   &Temp_offset_ram,&Temp_offset_100    ;           
            xor.w   #FXKEY+WRT,&FCTL1       ; Disable FLASH write 
            xor.w   #FXKEY+LOCK,&FCTL3      ; Lock FLASH memory
            bis.b   #PB_TEMP,&Calmode       ; Clear calmode
            clr.b   &P2IFG                  ; Clear P2 interrupt flags
            bis.b   #PB_VOLTAGE2+PB_TEMP+PB_VOLT,&P2IE  
return      eint
            ret                 
;------------------------------------------------------------------------------ 
Ref_cal  ;Calibration subroutine
;------------------------------------------------------------------------------ 
            bic.b   #PB_VOLTAGE2+PB_TEMP+PB_VOLT,&P2IE ; Disable PB interrupts
            bit.b   #PB_VOLTAGE2,&P2IN              ; Check if PB1 pressed
            jnz     RCAL2                   ; Jump -> PB1 not pressed
            sub.w   #10,&Refcal_ram
            jmp     return2            
RCAL2       bit.b   #PB_VOLT,&P2IN              ; Check if PB_TEMP pressed
            jz      RCAL3                   ; Jump -> PB1 not pressed     
            add.w   #10,&Refcal_ram                   
            jmp     return2                    
RCAL3       dint                            ; Disable global interrupt
            Call    #flash_erase             ; Erase info memory seg A
            xor.w   #FXKEY+WRT,&FCTL1       ; Enable FLASH write
            mov.w   &Refcal_ram,&Refcal_flash       ;
            mov.w   &Temp_offset_ram,&Temp_offset_100    ;           
            xor.w   #FXKEY+WRT,&FCTL1       ; Disable FLASH write 
            xor.w   #FXKEY+LOCK,&FCTL3      ; Lock FLASH memory
            bis.b   #PB_TEMP,&Calmode       ; Clear calmode
            clr.b   &P2IFG                  ; Clear P2 interrupt flags
            bis.b   #PB_VOLTAGE2+PB_TEMP+PB_VOLT,&P2IE  
            eint
return2     ret                 
;------------------------------------------------------------------------------ 
flash_erase ;Erase segment A of Info memory
;------------------------------------------------------------------------------ 
            xor.w   #FXKEY+LOCK,&FCTL3      ; Unlock FLASH for erase and write                                                         
Test_Busy1  bit.w   #BUSY,&FCTL3            ; Check BUSY flag
            jnz     Test_Busy1              ; Loop till not busy            
            mov.w   #FWKEY+ERASE,&FCTL1     ; enable segment erase
            clr.b   &1000h                  ; Perform a dummy write to seg A
Test_Busy2  bit.w   #BUSY,&FCTL3            ; Check BUSY flag
            jnz     Test_Busy2              ; Loop till not busy            
            ret                             ; Return from subroutine            
                                                                                                  
;------------------------------------------------------------------------------ 
Init_Sys   ;Configure modules and control registers 
;------------------------------------------------------------------------------ 
StopWDT     mov.w   #WDTPW+WDTHOLD,&WDTCTL  ; Stop Watchdog Timer
SetupFLL2   bis.b   #XCAP14PF,&FLL_CTL0     ; Set load capacitance for xtal

            mov.w   #10000,R15
FLL_Wait    dec.w   R15                     ; Delay for FLL to lock
            jnz     FLL_Wait		 
SetupPorts  mov.b   #000h,&P1OUT            ; Reset port outputs
            mov.b   #0FFh,&P1DIR            ; Set port to outputs
            bis.b   #BIT2,&P1SEL            ; Set for Timer A1 output
            mov.b   #000h,&P2OUT            ; Reset port outputs
            mov.b   #0FFh,&P2DIR            ; Set port to outputs
            bic.b   #PB_VOLTAGE2+PB_TEMP+PB_VOLT,&P2DIR; For push button inputs
            mov.b   #PB_VOLTAGE2+PB_TEMP+PB_VOLT,&P2IES; Interrupt edge select for push buttons
            clr.b   &P2IFG                  ; Clear pending P2 interrupts
            mov.b   #PB_VOLTAGE2+PB_TEMP+PB_VOLT,&P2IE; Enable P2.1,P2.2 & P2.3 interrupts
            mov.b   #000h,&P3OUT            ; Reset port outputs
            mov.b   #0FFh,&P3DIR            ; Set port to outputs
            mov.b   #004h,&P4OUT            ; For push button pull-up
            mov.b   #0FFh,&P4DIR            ; Set unused port to outputs              
            mov.b   #0FCh,&P5SEL            ; Set Rxx and COM pins for LCD
            mov.b   #000h,&P5OUT            ; Reset port outputs
            mov.b   #0FFh,&P5DIR            ; Set port to outputs
            mov.b   #019h,&P6SEL            ; Disable i/p buffer for P6.0,3,4
            mov.b   #020h,&P6DIR            ; Set unused pins to outputs
            mov.b   #000h,&P6OUT            ; Reset port outputs
            call    #ClearLCD
SetupFlash  xor.w   #FXKEY+FN2+FN1+FN0,&FCTL2
                                            ; Set FLASH timing generator 447.5Khz                       
SetupBT     mov.b   #BTFRFQ1+BTDIV+BT_fCLK2_DIV32,&BTCTL; 250ms Int, LCD freq
SetupADC1   mov.w   #ADC12ON+REFON+REF2_5V,&ADC12CTL0;ADC12, Ref ON, Int 2.5V
            mov.w   #SHP+CONSEQ_2+SHS_3,&ADC12CTL1; TB1 sampling timer,
                                            ; ADC12OSC=ADC12CLK
                                            ; Single channel repeated sequence           
            bis.b   #SREF_1,&ADC12MCTL0     ; Int reference
            eint                            ; enable global interrupts
            mov.w   #1500,&TBCCR0           ; Delay to allow Ref to settle
            bis.w   #CCIE,&TBCCTL0          ; Compare-mode interrupt.
            mov.w   #TBCLR+MC_1+TBSSEL_1,&TBCTL; up mode, ACLK
            bis.w   #LPM3,SR                ; Enter LPM3
            bic.w   #CCIE,&TBCCTL0          ; Disable timer interrupt.             
            mov.w   #0001h,&ADC12IE         ; Enable AD12IE.0        
            mov.w   #318,&TBCCR0            ; load period register for sampling
            mov.w   #159,&TBCCR1            ; Trigger for ADC12 Sampling
            mov.w   #OUTMOD_3,&TBCCTL1      ; Set/reset output            
            ret                             ;

;------------------------------------------------------------------------------ 
ClearLCD                                    ; Clear LCD
;------------------------------------------------------------------------------ 
            mov.w   #18,R15                 ; 18 LCD memory bytes to clear
Clear1      mov.b   #0,LCDM1(R15)           ; Write zeros in LCD RAM locations
            dec.w   R15                     ; All LCD mem clear?
            jc      Clear1                  ; More LCD mem to clear, use JC
SetupLCD    mov.b   #LCDON+LCD4MUX+LCDSG0_3,&LCDCTL ;4 mux LCD, segments = 0 - 23
            ret
;------------------------------------------------------------------------------ 
bin2bcd;     subroutine for converting 32bit binary to BCD
       ;     input 32bit binary, output 32bit BCD
;------------------------------------------------------------------------------ 
             mov.w  #32,r9                  ; 16-bits to be converted
             clr.w  r8                      ; Clear working register r8
             clr.w  r7                      ; Clear working register r7
L$1          rla.w  DIGITSLO
             rlc.w  DIGITSHI                     
             dadd.w r7,r7                   ; Decimal add result
             dadd.w r8,r8
             dec.w  r9                      ; Check if 32-bits done
             jnz    L$1
             mov.w  r7,DIGITSLO             ; Result to DIGITSLO
             mov.w  r8,DIGITSHI             ; Result to DIGITSHI
             ret                            ;                                            

;------------------------------------------------------------------------------ 
Display
Display_4  ;Subroutine to Display BCD digits in DIGITS 
           ;CPU Registers used R15, R14, R13 and not saved
;------------------------------------------------------------------------------ 
                        
            cmp.b   #0h,&mode               ; Check if voltage2 mode
            jnz     DisplayTemp             ; 
DisplayV    mov.w   #11,R15                 ; Start with LCDM12
DigitVLo    mov.b   DIGITSLO,R7             ; LSDigit value moved to R7
            and.b   #0Fh,R7                 ; R7 = Nibble now
            mov.b   LCD_Tab(R7),LCDM1(R15)  ; write the value to LCD memory
            rra.w   DIGITSLO                ; Right Shift to get next Digit 
            rra.w   DIGITSLO                ; Right Shift 
            rra.w   DIGITSLO                ; Right Shift 
            rra.w   DIGITSLO                ; Right Shift
            dec.w   R15                     ; Next LCD location
            cmp.w   #7h,R15                 ; x.xxxx (8.9 10 11 12)-->Display LCD Memory Map
            jnz     DigitVLo                ; If 7, DIGITSHI to be displayed
DigitVHi    mov.b   DIGITSHI,R7             ; LSDigit value moved to R7
            and.b   #0Fh,R7                 ; R7 = Nibble now
            mov.b   LCD_Tab(R7),LCDM1(R15)  ; Write the value to LCD
            bis.b   #DP, LCDM9              ; Place the decimal point
            ret
            
DisplayTemp cmp.b   #1h,&mode               ; Check if Temperature mode
            jnz     DisplayuV
            mov.w   #11,R15                 ; Start with LCDM12
DigitTLo    mov.b   DIGITSLO,R7             ; LSDigit value moved to R7
            and.b   #0Fh,R7                 ; R7 = Nibble now
            mov.b   LCD_Tab(R7),LCDM1(R15)  ; Write the value to LCD memory
            rra.w   DIGITSLO                ; Right Shift to get next Digit 
            rra.w   DIGITSLO                ; Right Shift 
            rra.w   DIGITSLO                ; Right Shift 
            rra.w   DIGITSLO                ; Right Shift
            dec.w   R15                     ; Next LCD location
            cmp.w   #7h,R15                 ; xxx.xxDF (6 7 8.9 10 11 12)
            jnz     DigitTLo                ; If 7, DIGITSHI to be displayed
DigitTHi    mov.b   DIGITSHI,R7             ; LSD value moved to R7
            and.b   #0Fh,R7                 ; R7 = Nibble now
            mov.b   LCD_Tab(R7),LCDM1(R15)  ; Write the value to LCD memory
            rra.w   DIGITSHI                ; Right Shift 
            rra.w   DIGITSHI                ; Right Shift 
            rra.w   DIGITSHI                ; Right Shift 
            rra.w   DIGITSHI                ; Right Shift
            dec.w   R15                     ; Next LCD location
            cmp.w   #4h,R15                 ; If 4, all digits are displayed
            jnz     DigitTHi
            bis.b   #DP, LCDM9              ; Place the decimal point
            ret         
                        
DisplayuV   mov.w   #11,R15                 ; Start with LCDM12
DigituVLo   mov.b   DIGITSLO,R7             ; LSDigit value moved to R7
            and.b   #0Fh,R7                 ; R7 = Nibble now
            mov.b   LCD_Tab(R7),LCDM1(R15)  ; Write the value to LCD memory
            rra.w   DIGITSLO                ; Right Shift to get next Digit 
            rra.w   DIGITSLO                ; Right Shift 
            rra.w   DIGITSLO                ; Right Shift 
            rra.w   DIGITSLO                ; Right Shift
            and.w   #00FFFh,DIGITSLO        ; Clear MS Nibble
            dec.w   R15                     ; Next LCD location
            cmp.w   #7h,R15                 ; x.xxxxxx (6. 7 8 9 10 11 12)
            jnz     DigituVLo               ; If 7, DIGITSHI to be displayed
DigituVHi   mov.b   DIGITSHI,R7             ; LSD value moved to R7
            and.b   #0Fh,R7                 ; R14 = Nibble now
            mov.b   LCD_Tab(R7),LCDM1(R15)  ; Write the value to LCD memory
            rra.w   DIGITSHI                ; Right Shift 
            rra.w   DIGITSHI                ; Right Shift 
            rra.w   DIGITSHI                ; Right Shift 
            rra.w   DIGITSHI                ; Right Shift
            and.w   #00FFFh,DIGITSHI
            dec.w   R15                     ; Next LCD location
            cmp.w   #4h,R15                 ; If 4, all digits are displayed
            jnz     DigituVHi           
            bis.b   #DP, LCDM7              ; Place the decimal point
            ret                    		 
;------------------------------------------------------------------------------ 
TBX_ISR;    Common ISR for CCR0-4 and overflow
;------------------------------------------------------------------------------ 
            add.w   &TBIV,PC                ; Add TB interrupt offset to PC
            jmp     TBCCR0_ISR              ; CCR0 
            reti                            ; CCR1
            reti                            ; CCR2
            reti                            ; CCR3
            reti                            ; CCR4
TB_over     reti                            ; Timer_B overflow

TBCCR0_ISR  clr     &TBCTL                  ; Clear Timer_B control registers
            mov.w   #GIE,0(SP)              ; Exit LPMx, interrupts enabled             
            reti                            ;            
;------------------------------------------------------------------------------ 
BT_ISR        
;------------------------------------------------------------------------------ 
done        bic.w   #ENC,&ADC12CTL0         ; Stop ADC conversions
            
            mov.w   #GIE,0(SP)              ; Exit LPMx, interrupts enabled
            reti                            ;		 
                                               
;------------------------------------------------------------------------------ 
ADC_ISR       
;------------------------------------------------------------------------------ 
calavg      sub.w   AVGBuffer(R5),&movingAVGLo
            subc.w  #0h,&movingAVGHi                       
            add.w   &ADC12MEM0,&movingAVGLo
            addc.w  #0h, &movingAVGHi
            mov.w   &ADC12MEM0,AVGBuffer(R5)
            incd.w  R5
            cmp.w   #0200h,R5
            jnz     L8
            mov.w   #0h,R5           
L8          mov.w   &movingAVGLo,R7        ; eg 0x1FFFE, Lo=0xFFFE
            
            mov.w   &movingAVGHi,R8        ; Hi = 0x0001
            
            swpb    R8                     ; R8=0x0100
            rra.w   R7                     ; Right Shift 
            rra.w   R7                     ; Right Shift 
            rra.w   R7                     ; Right Shift 
            rra.w   R7                     ; Right Shift, R7=0FFF
            and.w   #00FFFh,R7             ; eliminate MSbyte,eg 8000>>4=F800,not 0800
            
            rla.w   R8
            rla.w   R8
            rla.w   R8
            rla.w   R8                     ; R8=0x1000
            
            add.w   R8,R7                  ; R7=0x1000+0x0FFF=0x1FFF
            mov.w   R7,&ADCresult
            reti                            ;

;------------------------------------------------------------------------------ 
P2_ISR
;------------------------------------------------------------------------------ 
            mov.w   #15000,R15
debounce    dec.w   R15                      ; Delay for switch debounce
            jnz     debounce
            bis.b   #PB_VOLTAGE2,&P2IE       ; Enable PB1 interrupt
            bic.b   #01h,&P1OUT              ; LED off
            bit.b   #02h,&P2IFG
            jnz     VOLT2_IN
            mov.b   #0h,&mode
            
VOLT2_IN    bit.b   #PB_VOLTAGE2,&P2IFG     ; Check if PB1 is pressed
            jz      TEMP_IN                 ; Jump if not pressed to TEMP_IN
            mov.b   #0h, &mode
            mov.b   #0h,&Temp_mode
            cmp.b   #1h, &notlpm
            jz      exitLPM
            mov.b   #1h,&notlpm
            clr.b   &P2IFG                  ; Clear P2 interrupt flags
            eint
            bic.b   #LCDON,&LCDCTL
            bis.w   #LPM4,SR                ; Enter LPM4    
exitLPM     mov.w   #GIE,0(SP)              ; Exit LPMx, interrupts enabled
            mov.b   #0h,&notlpm
            clr.b   &P2IFG                  ; Clear P2 interrupt flags
            mov.w   #voltage2,2(SP)         ; Entry on exit
            reti                            ;

TEMP_IN     mov.b   #240,&Time_out          ; Max time Temp_mode or DVM_mode
            bit.b   #PB_TEMP,&P2IFG         ; Check if PB_TEMP is pressed
            jz      VOLT_IN                 ; Jump if not pressed to VOLT_IN
            mov.b   #1h, &mode
            cmp.b   #1h, &Temp_mode
            jz      chgTmpMd
            mov.b   #1h,&Temp_mode
            jmp     moveon
chgTmpMd    mov.b   #0h,&Temp_mode
moveon      clr.b   &P2IFG                  ; Clear P2 interrupt flags
            mov.w   #temperature,2(SP)      ; Entry on exit
            reti                            ;

VOLT_IN     mov.b   #2h, &mode
            mov.b   #0h,&Temp_mode
            clr.b   &P2IFG                  ; Clear P2 interrupt flags 
            mov.w   #voltage,2(SP)          ; Entry on exit
            reti                            ;

;------------------------------------------------------------------------------ 
;              COMMON    INTVEC              ; MSP430xG43x Interrupt vectors             
;------------------------------------------------------------------------------ 
              .sect   ".reset"
RESET_VEC     .short  RESET                 ; POR, ext. Reset, Watchdog

              .sect   ".int13"              ; Timer_BX Vector
TIMB_VEC      .short  TBX_ISR               ;

              .sect   ".int07"
ADC12_VEC     .short   ADC_ISR              ; ADC12 ISR 
              
              .sect   ".int01"
P2_VEC        .short   P2_ISR               ; Port 2 ISR
              
              .sect   ".int00"
BT_VEC        .short   BT_ISR               ; Basic Timer ISR

              .END
;******************************************************************************     
