//******************************************************************************
// THIS PROGRAM IS PROVIDED "AS IS". TI MAKES NO WARRANTIES OR
// REPRESENTATIONS, EITHER EXPRESS, IMPLIED OR STATUTORY, 
// INCLUDING ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS 
// FOR A PARTICULAR PURPOSE, LACK OF VIRUSES, ACCURACY OR 
// COMPLETENESS OF RESPONSES, RESULTS AND LACK OF NEGLIGENCE. 
// TI DISCLAIMS ANY WARRANTY OF TITLE, QUIET ENJOYMENT, QUIET 
// POSSESSION, AND NON-INFRINGEMENT OF ANY THIRD PARTY 
// INTELLECTUAL PROPERTY RIGHTS WITH REGARD TO THE PROGRAM OR 
// YOUR USE OF THE PROGRAM.
//
// IN NO EVENT SHALL TI BE LIABLE FOR ANY SPECIAL, INCIDENTAL, 
// CONSEQUENTIAL OR INDIRECT DAMAGES, HOWEVER CAUSED, ON ANY 
// THEORY OF LIABILITY AND WHETHER OR NOT TI HAS BEEN ADVISED 
// OF THE POSSIBILITY OF SUCH DAMAGES, ARISING IN ANY WAY OUT 
// OF THIS AGREEMENT, THE PROGRAM, OR YOUR USE OF THE PROGRAM. 
// EXCLUDED DAMAGES INCLUDE, BUT ARE NOT LIMITED TO, COST OF 
// REMOVAL OR REINSTALLATION, COMPUTER TIME, LABOR COSTS, LOSS 
// OF GOODWILL, LOSS OF PROFITS, LOSS OF SAVINGS, OR LOSS OF 
// USE OR INTERRUPTION OF BUSINESS. IN NO EVENT WILL TI'S 
// AGGREGATE LIABILITY UNDER THIS AGREEMENT OR ARISING OUT OF 
// YOUR USE OF THE PROGRAM EXCEED FIVE HUNDRED DOLLARS 
// (U.S.$500).
//
// Unless otherwise stated, the Program written and copyrighted 
// by Texas Instruments is distributed as "freeware".  You may, 
// only under TI's copyright in the Program, use and modify the 
// Program without any charge or restriction.  You may 
// distribute to third parties, provided that you transfer a 
// copy of this license to the third party and the third party 
// agrees to these terms by its first use of the Program. You 
// must reproduce the copyright notice and any other legend of 
// ownership on each copy or partial copy, of the Program.
//
// You acknowledge and agree that the Program contains 
// copyrighted material, trade secrets and other TI proprietary 
// information and is protected by copyright laws, 
// international copyright treaties, and trade secret laws, as 
// well as other intellectual property laws.  To protect TI's 
// rights in the Program, you agree not to decompile, reverse 
// engineer, disassemble or otherwise translate any object code 
// versions of the Program to a human-readable form.  You agree 
// that in no event will you alter, remove or destroy any 
// copyright notice included in the Program.  TI reserves all 
// rights not specifically granted under this license. Except 
// as specifically provided herein, nothing in this agreement 
// shall be construed as conferring by implication, estoppel, 
// or otherwise, upon you, any license or other right under any 
// TI patents, copyrights or trade secrets.
//
// You may not use the Program in non-TI devices.
//******************************************************************************

//******************************************************************************
//  MSP430FG439 Demo - ADC12, Using the ADC12 to get higher resolution 
//
//  This code demonstrates oversampling and averaging on ADC12
//  Voltage and temperature are displayed on the LCD after oversampling
//  The routines used are: voltage (1 uV mode), voltage2 (100 uV mode) and 
//  temperature (.01 degree resolution). These modes are selected using the
//  push buttons on the board
//  The above functions and the calibration routine is explained in the 
//  documentation of this application report
//  SW1 = P2.1, SW2 = P2.2, SW3 = P2.3   Pull-up = P4.2
//
//  H.Grewal
//  Texas Instruments, Inc
//  July 2005
//  Updated for IAR Embedded Workbench Version: 2.21B
//******************************************************************************

#include <msp430xG43x.h>                    // Definition header file for MSP430FG439

#define PB_VOLTAGE2 (1 << 1)                // Push button 1: Voltage2 --P2.1
#define PB_TEMP (1 << 2)                    // Push button 2: Temperature --P2.2
#define PB_VOLT (1 << 3)                    // Push button 3: Voltage --P2.3
#define PB_IN P2IN
// The switches read as zero when pressed (and one when released).
#define VOLT2_IN (!(PB_IN & PB_VOLTAGE2))
#define TEMP_IN (!(PB_IN & PB_TEMP))
#define VOLT_IN (!(PB_IN & PB_VOLT))
#define PB_IE P2IE
#define PB_IFG P2IFG

enum {
 FALSE,                                     //  0
 TRUE                                       //  1
};
 
enum {
 VOLTAGE2,                                  //  0
 TEMPERATURE,                               //  1
 VOLTAGE                                    //  2
};

typedef unsigned int word;                  // Type definition for 'word'
typedef unsigned long words;                // Type definition for 'words'

// Function prototypes.
__interrupt void p2_isr(void);
__interrupt void bt_isr(void);
__interrupt void adc_isr(void);
__interrupt void tb0_isr(void);

void init_sys(void);                        // System initialization.
void pwmgen(void);
void temperature(unsigned char Cmode);      // Temperature F or C mode.
void voltage2(void);
void voltage(void);                         // Voltmeter mode.
void display_temp(words temp, unsigned char Cmode); // Display temperature (Fahrht: xxF, Celsius: xxC).
void display_volts(words millivolts);       // Display volts (x.xxx).
void display_value(words value);            // Display value (xxxx).
unsigned long  bin2bcd(words i);            // Hex word to bcd.
void clearLCD(void);                        // Clears the LCD memory
void check_cal(void);                       // Check to initiallize cal constant.
void Temp_cal(void);                        // Temp sensor calibration function.
void Ref_cal(void);                         // ADC12 Ref calibration function.
void flash_write(word* address, word data); // Write the (integer) data to the addressed flash. 
void flash_erase(word* address);            // Erase the addressed flash segment.

// Variables in RAM.
static unsigned char mode = VOLTAGE2;       // Initially display voltage.
static unsigned char not_lpmode = VOLTAGE2; // Initially display voltage.
static unsigned char temp_mode = TRUE;      // Initially display temperature in F: FALSE=C. TRUE= F.
static unsigned char notlowpower_mode = TRUE;
static unsigned char calmode = 0;
static word Refcal_ram;                     // ADC12 1.5V reference calibration variable.
static word Temp_slope_ram;                 // Temperature slope calibration variable.
static float Temp_offset_ram;               // Temperature offset calibration variable.

unsigned long ADCresult;                    // Result after oversampling & Averaging 
                                            // 256 samples from ADC12
                   
long double test;

char Time_out = 0;
char push_time_sub = 0;
char push_time_add = 0;

unsigned long movingAVG = 0;
unsigned int AVGBuffer [256] = {0};
unsigned int AVGpnt = 0; 

// Global "variables" (in flash-INFO).
// The following variables are allocated in the flash Information memory; they can be read directly, but the flash
// controller must be used to erase and write them. These variables are non-volatile.

    
#pragma DATA_SECTION(Refcal_flash,".infoA") // ADC12 reference calibration in Flash INFO memory.
static word Refcal_flash ;
         
#pragma DATA_SECTION(Temp_slope,".infoA")   // Temperature sensor slope calibration in Flash INFO memory.
static word Temp_slope ;
       
#pragma DATA_SECTION(Temp_offset,".infoA")  // Temperature sensor offset calibration in Flash INFO memory.
static word Temp_offset ;

// LCD segment definitions.
#define a  (1<<4)   //      aaa
#define b  (1<<5)   //     f   b
#define c  (1<<6)   //     f   b
#define d  (1<<7)   //      ggg
#define e  (1<<2)   //     e   c
#define f  (1<<0)   //     e   c
#define g  (1<<1)   //      ddd
#define DP (1<<3)

// LCD Character Generator (stored in Flash).
const unsigned char LCDTable[16] =
{
  a + b + c + d + e + f,                    // '0'
  b + c,                                    // '1'
  a + b + d + e + g,                        // '2'
  a + b + c + d + g,                        // '3'
  b + c + f + g,                            // '4'
  a + f + g + c + d,                        // '5'
  a + f + g + c + d + e,                    // '6'
  f + a + b + c,                            // '7'
  a + b + c + d + e + f + g,                // '8'
  f + a + b + g + c + d,                    // '9'
  a + b + c + e + f + g,                    // Displays "A"
  0x00,                                     // Displays  Blank
  a + d + e + f,                            // Displays "C"
  a + b + f + g,                            // Displays "degrees" o
  a + d + e + f + g,                        // Displays "E"
  a + e + f + g                             // Displays "F"
};

#undef a
#undef b
#undef c
#undef d
#undef e
#undef f
#undef g

void main(void)
{
  volatile unsigned int i;
  WDTCTL = WDTPW | WDTHOLD;                 // Disable the Watchdog.
  FLL_CTL0 |= XCAP14PF;                     // Set load capacitance for xtal.
  for (i = 0; i < 20000; i++);              // Delay - to allow watch crystal to stabilize.
  init_sys();                               // System Initialization
  check_cal();                              // Check if INFO has calibration constants
  Refcal_ram = Refcal_flash;                // Register RAM copy of Vref from Flash
  Temp_slope_ram = Temp_slope;              // Register RAM copy of Temp sensor slope
  Temp_offset_ram = Temp_offset;            // Register RAM copy of Temp sensor offset
  calmode = P2IN & PB_TEMP;                 // Register Calibration mode(dual boot)
  
  ADCresult = 0;                            //Intialize 16-bit result  
  while (1)
  {  
      switch (mode)
    {
      case VOLTAGE2: voltage2(); break;     // Voltage2 mode
      case TEMPERATURE: temperature(temp_mode); break; // Temperature mode
      case VOLTAGE: voltage(); break;       // Micro-volt mode
      default: break;
    } 
  } // while (1)
} // main()


void init_sys(void)
{
  // Configure all unused port pins as outputs.
  P1DIR = 0xff;                             // Set port to outputs
  P1OUT = 0;                                // Reset port outputs
  P1SEL |= BIT2;                              // Set for Timer A1
  P2DIR = ~(PB_VOLTAGE2 | PB_TEMP | PB_VOLT); // Set port to inputs  
  P2OUT = 0;                                // Reset port outputs
  P2IES = (PB_VOLTAGE2 | PB_TEMP | PB_VOLT);// Interrupt edge select for push buttons 
  P2IFG = 0;                                // Clear pending P2 interrupts
  P2IE = PB_VOLTAGE2 | PB_TEMP | PB_VOLT;   // Enable P2.1,P2.2 & P2.3 interrupts
  P3DIR = 0xff;
  P3OUT = 0;
  P4DIR = 0xff;                             // Set unused port to outputs
  P4OUT = 0x04;                             // For push button pull-up
  P5SEL = 0xFC;                             // set Rxx and COM pins for LCD
  P5DIR = 0xff;
  P5OUT = 0;
  P6SEL = 0x01;                             // P6.0 = analog input
  P6DIR = 0x20;                             // set unused pins to outputs
  P6OUT = 0;
  clearLCD();
  
  FCTL2 ^= FXKEY | FN2 | FN1 | FN0;         // Set FLASH timing generator 447.5Khz
  
  BTCTL = BTFRFQ1 | BTDIV | BT_fCLK2_DIV32; // 250ms second interrupt, and LCD frequency.
  ADC12CTL0 = ADC12ON+REF2_5V+REFON;        // Setup ADC12, ref., sampling time
  ADC12CTL1 = SHP+CONSEQ_2+SHS_3;           // TB1 sampling timer,ADC12OSC=ADC12CLK
                                            // repeat Single channel
  ADC12MCTL0 |= SREF_1;                                                     
  _EINT();                                  // Enable interrupts.
  TBCCR0 = 1500;                            // Delay to allow Ref to settle
  TBCCTL0 |= CCIE;                          // Compare-mode interrupt.
  TBCTL = TBSSEL_1 | MC_1;                  // TBCLK = ACLK, Up mode.  
  LPM3;                                     // Wait for delay.
  TBCCTL0 &= ~CCIE;                         // Disable timer Interrupt
  ADC12IE = 0x0001;                         // Enable AD12IE to enter the ISR
                                            // after every conversions
  TBCCR0 = 318;                             // Init TBCCR0 w/ sample prd
  TBCCR1 = 159;                             // Trigger for ADC12 Sampling
  TBCCTL1 = OUTMOD_3;                       // Set/reset
} // sys_init()

void clearLCD(void)                         // Clears the LCD memory
{
   
    int i;

    for (i = 0; i < 20; LCDMEM[i++] = 0); 
    LCDCTL = LCDON+LCD4MUX+LCDSG0_3;        // Mux LCD, segments = 0 - 23

} // clearLCD()

//void voltage2(unsigned char not_lpmode)
void voltage2(void)
{

    ADC12CTL0 &= ~ENC;                      // Start conversion
    ADC12CTL0 &= ~SHT0_8;                   // Disable previous sampling time
    ADC12CTL0 |= SHT0_1;                    // Sampling time
    ADC12MCTL0 = INCH_3 + SREF_1;
    P6SEL |= 0x01;                          // Enable A/D channel A0  
    ADC12CTL0 |= ENC;                       // Start conversion
    TBCTL = TBCLR | MC_1 | TBSSEL_2;        // SMCLK, clear TBR, up mode
    P6OUT |= 0x20;
    IE2 |= BTIE;                            // Enable Basic Timer interrupt.
    LPM0;                                   // Wait for conversion completion.  
    P6OUT &= ~0x20;    
    IE2 &= ~BTIE;                           // Enable Basic Timer interrupt.
    test = ((long double)(ADCresult * Refcal_ram) / 65535);
    test = test * 10;                       // Voltage accuracy upto 100uV
    display_volts ((unsigned long) (test)); // Display the (milli) voltage.
    
} // voltage2()


void temperature(unsigned char Cmode)
{ 
  ADC12CTL0 &= ~ENC;                        // Stop conversion
  ADC12CTL0 &= ~SHT0_1;
  ADC12CTL0 |= SHT0_8;                      // Sampling time
  ADC12MCTL0 = INCH_10 + SREF_1;
  ADC12CTL0 |= ENC;                         // Start conversion.
  TBCTL = TBCLR | MC_1 | TBSSEL_2;          // SMCLK, clear TBR, up mode
  IE2 |= BTIE;                              // Enable Basic Timer interrupt.
  LPM0;                                     // Wait for conversion completion.  
  P6OUT &= ~0x20;    
  IE2 &= ~BTIE;                             // Enable Basic Timer interrupt.
  display_temp(ADCresult, Cmode);
} // temperature()

void voltage(void)
{
  if (calmode == 0)
    {
     Ref_cal();
    }
    else
    {
     Refcal_ram = Refcal_flash;
    }
  ADC12CTL0 &= ~ENC;                        // Start conversion
  ADC12CTL0 &= ~SHT0_8;                     // Disable previous sampling time
  ADC12CTL0 |= SHT0_1;                      // Sampling time
  ADC12MCTL0 = INCH_3 + SREF_1;
  P6SEL |= 0x01;                          // Enable A/D channel A0
  ADC12CTL0 |= ENC;                         // Start conversion
  TBCTL = TBCLR | MC_1 | TBSSEL_2;          // SMCLK, clear TBR, up mode  
  IE2 |= BTIE;                              // Enable Basic Timer interrupt.
  LPM0;                                     // Wait for conversion completion.  
  P6OUT &= ~0x20;    
  IE2 &= ~BTIE;                             // Enable Basic Timer interrupt.

  test= ((long double)(ADCresult * Refcal_ram) / 65535);
  
  test = test * 1000; 

  display_volts ((unsigned long) (test));   // Display the (micro) voltage.
} // voltage()

void display_temp(words temp, unsigned char Cmode)// Display temperature Fahrenheit
{  
   word sign;
   
   double tempF_final, tempC_final;         // Variables for temperature in F and C.
   long tempF, tempC ;
   
   if (calmode == 0)                        // Check if Cal mode
  {
    Temp_cal();                             // Run calibration function
    Cmode = FALSE;                          // Force display to deg F mode
  }
  else
  {
    Temp_slope_ram = Temp_slope;            // Make RAM copy from Flash (calibrated value)
    Temp_offset_ram = Temp_offset;          // Make RAM copy from Flash (calibrated value)
  } 
    tempF_final = ((long double)(temp * Temp_slope_ram) / 65535 - Temp_offset_ram);
    tempF = tempF_final*100;
   
   if (Cmode) 
   {
     tempC_final = ((long double)(tempF_final - 32) * 5/9);
     tempC= tempC_final * 100;
     
     if (tempC > 2944) 
     {
       P1OUT |= 0x01;                       // Turn-on LED.
     }
     else
     {
       P1OUT &= ~0x01;                      // Turn-off LED.
     }   
     if (tempC < 0 ) 
     {
       sign = 1;
       tempC= ~tempC + 1;                   // Make tempC positive (using simple 2's complement operation).
     }
     else
     {
       sign = 0;
     }
     display_value(((bin2bcd(tempC)) << 8) + 0x00dc); // Display temperature degrees C
 
   } 
   else
   {
 
     if (tempF > 8500) 
     {
       P1OUT |= 0x01;                       // Turn-on LED.
     }
     else
     {
       P1OUT &= ~0x01;                      // Turn-off LED.
     }   
     if (tempF < 0) 
     {
       sign = 1;
       tempF = ~tempF + 1;                  // Make tempF positive (using simple 2's complement operation).
     }
     else
     {
       sign = 0;
     }
     display_value(((bin2bcd(tempF)) << 8) + 0x00df); // Display temperature degrees F.
     
   } 
   if (sign)
   {
     LCDM4 |= 0x40;                         // Display "-".
   }
   else
   {
     LCDM4 &= ~0x40;                        // Clear "-".
   } 
     Time_out --;
   
   if (Time_out == 0)
   {
     P2IFG |= PB_VOLTAGE2;                  // Enable PB_VOLTAGE2 intterupt after 60 secs
   }
} // display_temp(temp, Cmode)

void display_volts(words millivolts)        // Display volts (v.vvvvvv) or (v.vvvv)
{
  display_value((bin2bcd(millivolts)));
  Time_out --;                                

   if (Time_out == 0)
   {
     P2IFG |= PB_VOLTAGE2;                  // Enable PB_VOLTAGE2 intterupt after 60 secs
   }
} // display_volts(millivolts)

void display_value(words value)
{
  unsigned int Digit;
  clearLCD();
  if (mode == VOLTAGE)
  {
    for (Digit=0; Digit < 7; Digit ++)
    {
     *(unsigned char *)(&LCDM12 - Digit) = LCDTable[value & 0x0F];
     value = value >> 4;
    }
    LCDM7 |= DP;
  }
  else if (mode == TEMPERATURE)
  {
    for (Digit=0; Digit < 6; Digit ++)
    {
     *(unsigned char *)(&LCDM12 - Digit) = LCDTable[value & 0x0F];
     value = value >> 4;
    }   
     LCDM9 |= DP;                           // Display "."
   }
   else
   {
     for (Digit=0; Digit < 5; Digit ++)
     {
      *(unsigned char *)(&LCDM12 - Digit) = LCDTable[value & 0x0F];
      value = value >> 4;
     }   
     LCDM9 |= DP;
   }
}   // display_value(value)

unsigned long bin2bcd(words i)              // Convert hex word to BCD.
{
  unsigned long bcd = 0;
  int j = 0;


  while (i > 9)
  {
    bcd |= ((i % 10) << j);
    i /= 10;
    j += 4;
  } // bin2bcd(i)

  return (bcd | (i << j));
}

void check_cal(void)                        // Check if INFO memory blank or contains calibration constants
{
  if (Refcal_flash == 0xffff)               // Check if Flash INFO Refcal erased?
  {
    flash_write(&Refcal_flash, 2500);       // Intialize with ideal Vref value 2500mV
    flash_write(&Temp_slope, 1269);         // Intialize ideal temperature diode slope for deg F .
    flash_write(&Temp_offset, 468);         // Intialize ideal temperature diode offset for deg F .    
    
  }  // check_cal()
}
void Temp_cal(void)                         // Temperature sensor calibration 
{ 
  PB_IE = 0;                                // Disable push button interrupts.  
  P1OUT = 1;                                // Toggle LED indicating TempCal
  
  if ((VOLT2_IN) && (TEMP_IN))
  {
    push_time_add ++;
    if (push_time_add == 3)
    {
     push_time_add = 0;                     // was skipped and needs to be reached again
    }
  }
  
  if (VOLT2_IN)                             // If VOLT2 button pressed decrement Temp offset
  {
    if (push_time_add == 0)
    Temp_offset_ram = Temp_offset_ram - .01;// Change the 2nd digit after decimal
    
    else if (push_time_add == 1)
    Temp_offset_ram = Temp_offset_ram - .1; // Change the 1st digit after decimal
    
    else
    Temp_offset_ram--;                      // Change the Temp by 1 degree F
  }         
  if (TEMP_IN)                              // If Temp button pressed increment Temp offset
  {
    if (push_time_add == 0)
    Temp_offset_ram = Temp_offset_ram + .01;
    
    else if (push_time_add == 1)
    Temp_offset_ram = Temp_offset_ram + .1;
    
    else
    Temp_offset_ram++;

  }
    Refcal_ram = Refcal_flash;              // Make RAM copy of Vref cal value
    Temp_slope_ram = Temp_slope;            // Make RAM copy of Temp slope value
  if (VOLT_IN)                              // If Volt button pressed store cal in Flash
  { 
    flash_erase(&Refcal_flash);             // Erase Flash INFO segment A. 
    flash_write(&Refcal_flash, Refcal_ram); // Write cal data to Refcal.
    flash_write(&Temp_offset, Temp_offset_ram);  
    flash_write(&Temp_slope, Temp_slope_ram);  
    calmode |= PB_TEMP;                     // Clear cal mode.
    push_time_add = 0;
    PB_IFG = 0;                             // Clear any pending push button int flags.
    PB_IE = PB_VOLTAGE2 | PB_TEMP | PB_VOLT;// Enable push button interrupts.
  }
}// void Temp_cal()
 
void Ref_cal(void)                          // ADC12 Vref calibration
{ 
  PB_IE = 0;                                // Disable push button interrupts.
  P1OUT ^= 1;

  if (VOLT2_IN)                             // If VOLT2 button pressed decrement cal
  {
    Refcal_ram--;
  }         
  if (TEMP_IN)                              // If Temp button pressed increment cal
  {
    Refcal_ram++;
  }
  if (VOLT_IN)                              // If Volt button pressed store cal in Flash
  { 
    flash_erase(&Refcal_flash);             // Erase Flash INFO segment of Refcal.
    flash_write(&Refcal_flash, Refcal_ram); // Write cal data to Refcal.
    flash_write(&Temp_offset, Temp_offset_ram); // Write Temp cal data. 
    flash_write(&Temp_slope, Temp_slope_ram);  
    calmode |= PB_TEMP;                     // Clear cal mode.
    PB_IFG = 0;                             // Clear any pending push button int flags.
    PB_IE = PB_VOLTAGE2 | PB_TEMP | PB_VOLT;// Enable push button interrupts.
  } 
} // Ref_cal()

void flash_write(word* address, word data)  // Write the (integer) data to the addressed flash.
{
  word genint = _get_SR_register() & GIE;   // Disable interrupts.

  _BIC_SR(GIE);

  FCTL3 = FWKEY;                            // Unlock the flash.
  FCTL1 = FWKEY | WRT;                      // Enable flash write.
  *address = data;                          // Write the data to the flash.
  FCTL1 = FWKEY;                            // Disable flash write.
  FCTL3 = FWKEY | LOCK;                     // Lock the flash.  
  _BIS_SR(genint);                          // Restore interrupts (to previous state).
}// flash_write(word* address, data)

void flash_erase(word* address)             // Erase the addressed flash segment.
{
  word genint = _get_SR_register() & GIE;   // Disable interrupts.
  _BIC_SR(GIE);
  
  FCTL3 = FWKEY;                            // Unlock the flash.
  FCTL1 = FWKEY | ERASE;                    // Enable flash segment erase.
  *address = 0;                             // Erase the flash segment.
  FCTL1 = FWKEY;                            // Disable flash segment erase.
  FCTL3 = FWKEY | LOCK;                     // Lock the flash.
  _BIS_SR(genint);                          // Restore interrupts (to previous state).
}// flash_erase(word* address, data)
  
// Interrupt Service Routines.

PORT2_ISR(p2_isr)
__interrupt void p2_isr(void)
{
  volatile unsigned int i;
  for (i = 0; i < 4000; i++);               // Debounce
  P2IE |= PB_VOLTAGE2;                      //
  P1OUT &= ~0x01;                           // Turn off LED (if on).
  
   
  if (P2IFG == 0x02)                        // Check for the 60 sec interrupt
  {
     mode = VOLTAGE2;
     temp_mode = TRUE;
  }
  if (VOLT2_IN)
  {
    mode = VOLTAGE2;
    temp_mode = TRUE;
    notlowpower_mode = notlowpower_mode ? FALSE : TRUE; // Toggle between V and LPM4 after second button push.
    if (notlowpower_mode)
     LPM4_EXIT;
    else
    {
     PB_IFG = 0;                            // Clear all button interrupts.
     _BIS_SR(GIE);                          // Restore interrupts (to previous state).
     LCDCTL &= ~LCDON;
     LPM4;
    }
  }
  else
  {
    Time_out = 240;
    if (TEMP_IN)
    {
      mode = TEMPERATURE;
      temp_mode = temp_mode ? FALSE : TRUE; // Toggle between F and C after second button push.
     }
    else
     {
       if (VOLT_IN)
       {
         mode = VOLTAGE;
         temp_mode = TRUE;
       }
     }
  }
  PB_IFG = 0;                               // Clear all button interrupts.
}// p2_isr()

BASICTIMER_ISR(bt_isr)
__interrupt void bt_isr(void)
{ 
  LPM0_EXIT;                                // Exit LPM0 mode on return
}// bt_isr()


ADC12_ISR(adc_isr)
__interrupt void adc_isr(void)
{
   movingAVG -= AVGBuffer[AVGpnt];
   movingAVG += ADC12MEM0;
   AVGBuffer[AVGpnt++] = ADC12MEM0;
   if (AVGpnt == 256)
     AVGpnt = 0;
   
   ADCresult = movingAVG >> 4;
}// adc_isr()

TIMERB0_ISR(tb0_isr)
__interrupt void tb0_isr(void)
{
  TBCTL = 0;
  LPM3_EXIT;                                // Exit LPM3 on return
}// tb0_isr()

